@@ -166,6 +166,14 @@ class IndexedSet(set):
             if item not in common:
                 self.remove(item)
 
+    def difference(self, *args):
+        """\
+        Return the difference of the set with other iterables.
+        """
+        result = self.copy()
+        result.difference_update(*args)
+        return result
+
     def difference_update(self, *args):
         """\
         Update the set with the difference of itself and other iterables.
@@ -175,6 +183,14 @@ class IndexedSet(set):
         for item in common:
             self.discard(item)
 
+    def symmetric_difference(self, *args):
+        """\
+        Return the symmetric difference of the set with other iterables.
+        """
+        result = self.copy()
+        result.symmetric_difference_update(*args)
+        return result
+
     def symmetric_difference_update(self, *args):
         """\
         Update the set with the symmetric difference of itself and other
